#ifndef __cancelableWindow
#define __cancelableWindow

// -------------------------------- includes ---------------------------
#include <qwidget.h>
#include "cancelablewindowfactory.h"
//----------------------------------------------------------------------

/**
   A window in which you can cancel your edits.

   The CancelableWindow widget offers you a popup window, where the edits
   can be canceled.

   When you call the @ref show or the @ref exec function the window will be
   visible. In this window an Ok button and a Cancel button
   exists. Pressing the Cancel button will revert the widgets of the window
   to their values before the window was show.

   To use this widget, you must create a class which inherit from the @ref
   CancelableWindowFactory class. This class must implement the @ref
   CancelableWindowFactory::create method to return a QWidget with the
   content you want in the pop up window.

   For this extra work you get for free a cancel button on your page, which
   will revert the content of the QWidget to its state before the window
   was shown.

   @author Jesper Kjr Pedersen <blackie@ifad.dk> 
   @version 0.1
 **/
class CancelableWindow : public QWidget
{

  Q_OBJECT

public:

  /**
     The content parameter should be used like in the following example:
     <pre>
     MyClass *content; // Note no initialization
     CancelableWindow *myWindow 
       = new CancelableWindow(myFact, &((QWidget *) content))
     </pre>
     @param factory The factory which will create the widget to be shown in
     the window.
     @param content A pointer to a QWidget pointer containing the content
     of the popup menu. Note that the CancelableWindow will take care of
     initializing it.
  **/
  CancelableWindow(CancelableWindowFactory *factory, QWidget **content, bool modal=false,
                   const char *name = 0, WFlags f = 0);
  
public slots:
  /**
     Shows the popup window. The user may hide the window again by pressing
     either the cancel button or the ok button.
  **/
  void show();

  /**
     Shows the popup menu as @ref show, but do not return until the window is closed.
  **/
  void exec();

signals:
  void windowClosed();
  void okButPressed();
  void cancelButPressed();
    
protected slots:
  void okButPushed();
  void cancelButPushed();

private:
  CancelableWindowFactory *myFactory;
  QWidget **myContent;  
  QWidget *contentBackup;  
  bool execing;
};


#endif // __cancelableWindow
