// ------------------------------- Description ----------------------------
// This is the ExtEntry Widget. You do not need to inheret from this class,
// your interface to it will be through the methods `append` and `elements'.
//
// To make an instance of the widget, you need to inherit the classes
// `ExtEntryFactory' and `ExtEntryEntry'.
// ------------------------------------------------------------------------
#ifndef __extentry
#define __extentry

// -------------------------------- includes ------------------------------
#include "extentryfactory.h"
#include "eee.h"
#include <qlist.h>
#include <qlayout.h>
#include <qscrollview.h>
//-------------------------------------------------------------------------

// EEE_list stands for ExtEntryEntry-list.
typedef QList<ExtEntryEntry> EEE_list ;
typedef QList<QWidget> WidgetList;

/**
 * The main class used to get an ExtEntry widget.
 
 * The Extentry widget consist of a sub-widget which is repeated a number
 * of times, it is up to the end user to determine the number of times the
 * sub widget is repeated, and he may require an additional copy simply by
 * pressing a ``More Entries'' button. Currently this button must be
 * inserted by the programmer using the ExtEntry widget, and connected to
 * the slot @ref addElement
 *
 * To use the ExtEntry widget you must create a class which is inherited
 * from the @ref ExtEntryFactory class. This new class must override the
 * function `create'. This function must return a freshly made instance of
 * the class @ref ExtEntryEntry (or a subclass of this). The ExtEntryEntry
 * instance is the one inserted into the ExtEntry widget (one instance for
 * each widget repeated in the ExtEntry widget).
 *
 * @author Jesper Kjr Pedersen <blackie@ifad.dk> 
 * @version 0.1
 **/
class ExtEntry : public QScrollView {

Q_OBJECT
  
public:
  /**
   * @param factory A factory used to generate the instances of ExtEntryEntry
   * class which is repeated in the ExtEntry
   * @param parent A pointer to the parent widget
   **/
  ExtEntry(ExtEntryFactory *factory, QWidget *parent = 0, const char *name = 0);

  /**
   * @return The elements in the extentry.
   **/
  EEE_list elements(); 
  void append(ExtEntryEntry *);

  /**
   * This method copy the data from the ExtEntry `from' to itself.
   * This is needed by the class @ref CopyFactory.
   *
   * @param from The ExtEntry to copy data from.
   **/
  void copyWidget(const ExtEntry *from);
  

public slots:
  
  /**
   * This slot should be connected to a button which lets the user know that
   * he may get more elements in this ExtEntry by pressing it. 
   * The button should be labeled ``More Entries'' or something similar.
   **/
  void addElement(); // Adds an empty element to the ExtEntry


protected slots:

  /**
   * When this slot is invoked then the Idx menu will be shown.
   * @param elm The ExtEntryEntry element where the Idx button was
   * pressed. This information is necessary to be able to calculate the
   * location where the drop down menu should be shown
   **/
  void showIndexList(ExtEntryEntry *elm);

protected:
  /**
   * Copies the element pointed to by which to the clipboard and removes it
   * from the interface.
   * This function is accessed from the class @ref CCP.
   * @param which A pointer to the element to cut.
   **/
  void cut(ExtEntryEntry *which);
  
  /**
   * Copies the element pointed to by which to the clipboard.
   * This function is accessed from the class @ref CCP.
   * @param which A pointer to the element to copied.
   **/
  void copy(ExtEntryEntry *which);

  /**
   * Paste the element from the clipboard to the ExtEntry
   * This function is accessed from the class @ref CCP.
   * @param which A pointer to the ExtEntryEntry element which the
   * clipboard element should be inserted before,.
   **/
  void paste(ExtEntryEntry *which);

  /**
   * Inserts an empty ExtEntryEntry into the widget.
   * @param which A pointer to the element which the new empty widget
   should be inserted before.
   **/
  void addElement(ExtEntryEntry *);

  friend class CCP;

  virtual void resizeEvent(QResizeEvent *);
  void addElemBefore(ExtEntryEntry *newElm, QWidget *existing);
  void insertElmIntoWidget(ExtEntryEntry *elm, ExtEntryEntry *after);
  void showWidget(ExtEntryEntry *elm);
  void delElem(QWidget *);
  void addElemBefore(ExtEntryEntry *newElm, ExtEntryEntry *existing);
  void updateClipperContent();
  int countEEE(WidgetList *);
  


private: 
  // Instance variables

  ExtEntryFactory *factory;
  WidgetList *elms;
  QPushButton *addBut;
  ExtEntryEntry *clipboard;
};


#endif /* extentry */

