#ifndef __fillout
#define __fillout

#include "filloutfactory.h"
#include <qmultilineedit.h>
#include <qevent.h>
#include <qlist.h>
class FillOutWidgetList;

typedef QList<FillOutFactory *> FillOutFactoryList;

/**
   The FillOut Widget is for combined text and token insertion.

   The FillOut widget is to be used where the end user should type in some
   text combined with some tokens. The easiest way to explain this is to
   give an example.

   Think of the prompt for the shell, say the bash shell. This prompt
   contains text combined with tokens identifying locations where some
   information about the current directory, the current user name, the
   date, or something similar should be inserted.

   The user is presented with a text box (single-lined or multi-lined)
   where he may insert ordinary text. When he presses the right mouse
   button a popup menu will present him with a listbox where each entry
   will either insert a token into the text box or bring up a window with
   additional configuration which will select a token.

   Turning back to our prompt example. The host name may be accessible as
   an element in the popup menu without further configuration. On the other
   hand, the current directory may be an example where an additional window
   is required. In this window the end user may select whether the current
   directory should be inserted with full path or just with the last three
   directories.

   To use the FillOut widget you will simply create an instance of the
   FillOut class. You do however need to inherit the @ref FillOutFactory
   class and inherit the @ref FillOutElm class once for each of the element
   in the listbox mentioned above.

   For an example of the use of this widget, see the example directory.

**/
class FillOut : public QMultiLineEdit {
public:

  /** 
      Type used to tell wether the FillOut widget should be single or multilined.
   **/
  enum DisplayType {SingleLine, MultiLine};

  /**
     Constructor for the FillOut widget.
     
     @param parent A pointer to the parent for the FillOut Widget
     @param fact A factory which should generate the tokens for the FillOut widget.
     @param tp indicate whether the FillOut widget should be single lined
     or multilined (just like QLineEdit and QMultiLineEdit)
  **/
  FillOut(QWidget *parent, FillOutFactory *fact, DisplayType tp = MultiLine, 
          const char *name = 0);

  /** 
      This method return a list of the elements in the FillOut.

      Text string are returned in instances of the class @ref FillText,
      while tokens are returned in instances of their @ref FillOutElm
      classes.
  **/
  FillList *elements();


protected:
  enum Direction {LEFT, RIGHT};

  virtual void mousePressEvent(QMouseEvent *event);
  virtual void keyPressEvent( QKeyEvent *event);
  virtual void leftButtonPressed(QMouseEvent *event);
  virtual void rightButtonPressed(QMouseEvent *event);
  virtual void newLine();
  virtual void insert(const QString &str, bool mark = false);
  virtual void cursorLeft(bool mark = false, bool wrap = true);
  virtual void cursorRight(bool mark = false, bool wrap = true);
  virtual void cursorUp(bool mark = false);
  virtual void cursorDown(bool mark = false);
  virtual void cursorWordBackward(bool mark = false);
  virtual void cursorWordForward(bool mark = false);
  virtual void pageUp(bool mark = false);
  virtual void pageDown(bool mark = false);
  virtual void killLine();
  virtual void backspace();
  virtual void del();
  FillOutWidgetList *delRegion(const QPoint& start, const QPoint& end,
                               bool returnCut, bool doDelete);
  virtual void copy();
  virtual void cut();
  virtual void paste();
  bool exec(FillOutElm *elm);
  void moveAwayFromToken(Direction dir, bool mark);
  QString region(const QPoint* start, const QPoint* end);

public:
  virtual QSize sizeHint() const;

private:
  void setMyMaxSize();

private: // instance variables
  FillOutFactory *factory;
  DisplayType myType;

  FillOutWidgetList *fillList;
  FillOutWidgetList *clipboard;
  QString clipboardText; // for validating if the text pasted in belongs to
                         // the text on the clipboard
};
#endif /* fillout */

