#ifndef __filloutelm
#define __filloutelm

#include <qwidget.h>
#include <qstring.h>
#include <qlist.h>

/** 
    Base class for the elements in the list returned by the @ref FillOut::elements method.

    The @ref generate method of this class is abstract, which ensures that
    every widget in the @ref FillList list has defined this method them
    self. In other word every subclass of @ref FillOutElm must define a
    generate method.
**/
class Fills {
public:
  
  /**
     This method must return a string representation of the given
     widget. This ensures that one can simply invoke the generate method of
     all widgets in the FillOut widget and concatenate the output, to get a
     string representation of the whole widget.

     It is easy to imagine a use of the FillOut widget where this does not
     make sense, but it seems very convenient for The Dotfile Generator,
     which will use this widget intensively.
   **/
  virtual QString generate() = 0;
  virtual ~Fills() {};
};


typedef QList<Fills> FillList;


/**
   Class used to return text part of the @ref FillOut widget by the @ref FillOut::elements method
**/
class FillText : public QString, public Fills {
public:
  FillText(QString str) : QString(str) {};
  virtual QString generate() { return *this; };
};

/**
   Base class for elements representing tokens in a @ref FillOut widget.

   To understand this widget, please read the description for the @ref
   FillOut widget first.

   The FillOutElm class is used for the tokens in the FillOut class. Each
   instance correspond to a token in the text box of the FillOut widget.

   A token may be one of two:
   @li A simple token without any configuration window connected to it.
   @li A complex token which is configured using a window, which appears
   when the user selects the token in the popup window.

   Each of these two types are created differently. 

   The first type is simply created by letting the factory (see @ref
   FillOutFactory) return an instance of the FillOutElm class with type
   @ref FillOutType::TextOnly.

   The other is created by inheriting the FillOutElm widget. This widget
   must build a widget with the configuration window required.

   Confused? Don't panic two examples of this widget is available in the
   Examples directory.
**/
class FillOutElm :public QWidget, public Fills {

  Q_OBJECT

private:
  FillOutElm() {} // Nobody should call this constructor!  

public:
  enum FillOutType {TextOnly, SubWidgets};

  /**
     Constructor for the FillOutElm Widget.
     @param parent A pointer to the parent for this widget.
     @param which The number given as argument to the @ref
     FillOutFactory::create method.
     @param text The text string which will be inserted into the text
     box. This might be overridden by the @ref text method.
     @param genText The text string returned in the @ref generate
     method. This may be overridden by the @ref generate method.
     @param tp The type of this FillOutElement.
  **/
  FillOutElm(QWidget *parent, int which, QString text="", QString genText = "",
             FillOutType tp=SubWidgets, const char *name = 0) : QWidget(parent, name) {
    txt = "[" + text + "]";
    generateText = genText;
    myType = tp;
    whichAmI = which;
  }
  
  /**
     returns the text to be insert into the text box
  **/
  virtual QString text() {
    return txt;
  }

  int which() {
    return whichAmI;
  }

  FillOutType type() {
    return myType;
  }
  
  /**
     See @ref Fills::generate
  **/
  virtual QString generate() { 
    return generateText;
  }

private: //instance variables
  FillOutType myType;
  QString txt;
  QString generateText;
  int whichAmI;
};

#endif /* filloutelm */

